unit uRelMan;

interface

uses
  classes
  , contnrs
  ;

type

  TRelMan = class(TObject)
  private

    FRels: TObjectList;

  public

    constructor Create;
    destructor Destroy; override;

    function FindObject(From, OnTo: TObject; RelID: integer): TObject;
    function FindObjects(From, OnTo: TObject; RelID: integer): TList;
    procedure AddRelationship(From, OnTo: TObject; RelID: integer);
    procedure RemRelationship(From, OnTo: TObject; RelID: integer);
    procedure DelObject(Obj: TObject);

  end;

function gRelMan: TRelMan;        // Simulate Singleton behaviour

implementation

var
  ugRelMan: TRelMan;

function gRelMan: TRelMan;
begin
  if not Assigned(ugRelMan) then
    ugRelMan := TRelMan.Create;
  Result := ugRelMan;
end;

type
  TRel = class(TObject)
  private

    FFrom: TObject;
    FOnTo: TObject;
    FID: integer;

  public

    constructor Create(From, Onto: TObject; ID: integer);

    property From: TObject read FFrom write FFrom;
    property OnTo: TObject read FOnTo write FOnTo;
    property ID: integer read FID write FID;

  end;

{ TRel }

constructor TRel.Create(From, Onto: TObject; ID: integer);
begin
  inherited Create;
  FFrom := From;
  FOnTo := OnTo;
  FID := ID;
end;

{ TRelMan }

procedure TRelMan.AddRelationship(From, OnTo: TObject; RelID: integer);
var
  lstTemp: TList;
begin
  lstTemp := nil;
  try { ... finally }
    lstTemp := FindObjects(From, OnTo, RelID);
    if (lstTemp.Count <= 0) then
      FRels.Add(TRel.Create(From, OnTo, RelID));
  finally
    lstTemp.Free;
  end;  { try ... finally }
end;

constructor TRelMan.Create;
begin
  inherited Create;
  FRels := TObjectList.Create(True);
end;

procedure TRelMan.DelObject(Obj: TObject);
var
  nRel: integer;
begin
  // Iterate through the list of relationships. Delete any that involve Obj.
  for nRel := Pred(FRels.Count) downto 0 do
  begin
    if (TRel(FRels.Items[nRel]).From = Obj) or
       (TRel(FRels.Items[nRel]).OnTo = Obj) then
    begin
      FRels.Delete(nRel);
    end;
  end;
end;

destructor TRelMan.Destroy;
begin
  FRels.Free;
  inherited;
end;

function TRelMan.FindObject(From, OnTo: TObject; RelID: integer): TObject;
var
  lstTemp: TList;
begin
  // Call FindObjects to get a list of related objects and return it. If no
  // relationship found, return nil. If From is nil, return the OnTo value. If
  // OnTo is nil, return the From value. If neither is nil, return a pointer to
  // the relationship object (can use it to test to see if a relationship
  // exists).
  Result := nil;
  lstTemp := nil;
  try { ... finally }
    lstTemp := FindObjects(From, OnTo, RelID);
    if (lstTemp.Count > 0) then
      Result := lstTemp.Items[0];
  finally
    lstTemp.Free;
  end;  { try ... finally }
end;

function TRelMan.FindObjects(From, OnTo: TObject; RelID: integer): TList;
var
  nRel: integer;
  relTemp: TRel;
begin
  Result := TList.Create;
  // Check all the relationships to see which ones match. Add matching ones to
  // the list. Note that either From or OnTo may be unassigned, in which case we
  // return a list of them. If both are assigned, return a list of relationships
  for nRel := 0 to Pred(FRels.Count) do
  begin
    relTemp := TRel(FRels.Items[nRel]);
    if ((not Assigned(From)) or (relTemp.From = From)) and
       ((not Assigned(OnTo)) or (relTemp.OnTo = OnTo)) and
       (relTemp.ID = RelID) then
    begin
      if not Assigned(From) then
        Result.Add(relTemp.From)
      else if not Assigned(OnTo) then
        Result.Add(relTemp.OnTo)
      else
        Result.Add(relTemp);
    end;
  end;
end;

procedure TRelMan.RemRelationship(From, OnTo: TObject; RelID: integer);
var
  nRel, i: integer;
  lstTemp: TList;
begin
  lstTemp := nil;
  // Check all the relationships to see if they match. If they do delete them.
  try { ... finally }
    lstTemp := FindObjects(From, OnTo, RelID);
    for nRel := 0 to Pred(lstTemp.Count) do
    begin
      i := FRels.IndexOf(lstTemp.Items[nRel]);
      if (i > -1) then
      begin
        FRels.Delete(i);
      end;
    end;
  finally
    lstTemp.Free;
  end;  { try ... finally }
end;

initialization

finalization
  ugRelMan.Free;

end.
